#!/usr/bin/env python

from ..base import Manager


class Serie(Manager):
    def __init__(self, client):
        super().__init__(client)
        self.serie_path = "/series"
        self.serie_id_path = "/series/{id}".format(id="{id}")

    def get_all_series(self):
        return self.client.http_get(self.serie_path)

    def update_serie(self, updated_serie_dict):
        return self.client.http_put(self.serie_path, json=updated_serie_dict)

    def delete_serie(self, id, delete_files, add_import_exclusion):
        params = {
            "deleteFiles": delete_files,
            "addImportExclusion": add_import_exclusion,
        }
        return self.client.http_delete(self.serie_id_path.format(id=id), params=params)



from ..base import Manager


class TV(Manager):
    def get_details(self, tv_id):
        path = "/tv/{}".format(tv_id)

        return self.client.http_get(path)

    def get_watch_providers(self, tv_id):
        path = "/tv/{}/watch/providers".format(tv_id)

        return self.client.http_get(path)



class RadarrTooManyVariablesException(Exception):
    pass


class RadarrInvalidIdSupplied(Exception):
    pass


class RadarrInvalidApiKey(Exception):
    pass


class RadarrMovieNotFound(Exception):
    pass


class RadarrValidationException(Exception):
    pass



import typer
from typing import Optional
from loguru import logger

from excludarr.modules.justwatch import justwatch
from excludarr.utils.config import Config
from excludarr.utils import output

app = typer.Typer()


@app.command(help="Get all the possible providers for your locale")
def list(
    locale: Optional[str] = typer.Option(
        None, "-l", "--locale", metavar="LOCALE", help="Your locale e.g: en_US."
    )
):
    # Check if locale is set on CLI otherwise get the value from the config
    if not locale:
        locale = config.locale

    justwatch_client = justwatch.JustWatch(locale)
    jw_providers = justwatch_client.get_providers()

    output.print_providers(jw_providers)


@app.callback()
def init():
    """
    Initializes the command. Reads the configuration.
    """
    logger.debug("Got radarr as subcommand")

    # Set globals
    global config
    global loglevel

    # Hacky way to get the current log level context
    loglevel = logger._core.min_level

    logger.debug("Reading configuration file")
    config = Config()



from loguru import logger
from rich.progress import Progress

import excludarr.modules.pysonarr as pysonarr
import excludarr.modules.pytmdb as pytmdb
import excludarr.utils.filters as filters

from excludarr.modules.justwatch import JustWatch
from excludarr.modules.justwatch.exceptions import JustWatchNotFound, JustWatchTooManyRequests


class SonarrActions:
    def __init__(self, url, api_key, locale, verify_ssl=False):
        logger.debug(f"Initializing PySonarr")
        self.sonarr_client = pysonarr.Sonarr(url, api_key, verify_ssl)

        logger.debug(f"Initializing JustWatch API with locale: {locale}")
        self.justwatch_client = JustWatch(locale)

    def _get_jw_serie_data(self, title, jw_entry):
        jw_id = jw_entry["id"]
        jw_serie_data = {}
        jw_imdb_ids = []
        jw_tmdb_ids = []

        try:
            logger.debug(f"Querying JustWatch API with ID: {jw_id} for title: {title}")
            jw_serie_data = self.justwatch_client.get_show(jw_id)

            jw_imdb_ids = filters.get_imdb_ids(jw_serie_data.get("external_ids", []))
            logger.debug(f"Got IMDB ID's: {jw_imdb_ids} from JustWatch API")

            jw_tmdb_ids = filters.get_tmdb_ids(jw_serie_data.get("external_ids", []))
            logger.debug(f"Got TMDB ID's: {jw_tmdb_ids} from JustWatch API")
        except JustWatchNotFound:
            logger.warning(f"Could not find title: {title} with JustWatch ID: {jw_id}")
        except JustWatchTooManyRequests:
            logger.error(f"JustWatch API returned 'Too Many Requests'")
            # TODO: Raise error so typer can abort properly

        return jw_serie_data, jw_imdb_ids, jw_tmdb_ids

    def _find_using_imdb_id(self, title, sonarr_id, imdb_id, fast, jw_query_payload={}):
        # Log the title and Sonarr ID
        logger.debug(
            f"Processing title: {title} with Sonarr ID: {sonarr_id} and IMDB ID: {imdb_id}"
        )

        # Log the JustWatch API call function
        logger.debug(f"Query JustWatch API with title: {title}")
        jw_query_data = self.justwatch_client.query_title(title, "show", fast, **jw_query_payload)

        for entry in jw_query_data["items"]:
            jw_id = entry["id"]
            jw_serie_data, jw_imdb_ids, _ = self._get_jw_serie_data(title, entry)

            # Break if the TMBD_ID in the query of JustWatch matches the one in Sonarr
            if imdb_id in jw_imdb_ids:
                logger.debug(f"Found JustWatch ID: {jw_id} for {title} with IMDB ID: {imdb_id}")
                return jw_id, jw_serie_data

        logger.debug(f"Could not find {title} using IMDB ID: {imdb_id}")
        return None, None

    def _find_using_tvdb_id(self, title, sonarr_id, tvdb_id, fast, jw_query_payload={}):
        # Log the title and Sonarr ID
        logger.debug(
            f"Processing title: {title} with Sonarr ID: {sonarr_id} and TVDB ID: {tvdb_id}"
        )

        # Log the JustWatch API call function
        logger.debug(f"Query JustWatch API with title: {title}")
        jw_query_data = self.justwatch_client.query_title(title, "show", fast, jw_query_payload)

        # Get TMDB ID from TMDB using the TVDB ID
        logger.debug(f"Trying to obtain the TMDB ID using TVDB ID: {tvdb_id} from TMDB API")
        tmdb_id = 0
        tmdb_find_result = self.tmdb.find.find_by_id(tvdb_id, "tvdb_id").get("tv_results", [])
        if tmdb_find_result:
            # Default to 0 if no ID is found
            tmdb_id = int(tmdb_find_result[0].get("id", 0))

        if tmdb_id != 0:
            for entry in jw_query_data["items"]:
                jw_id = entry["id"]
                jw_serie_data, _, jw_tmdb_ids = self._get_jw_serie_data(title, entry)

                # Break if the TMBD_ID in the query of JustWatch matches the one in Sonarr
                if tmdb_id in jw_tmdb_ids:
                    logger.debug(f"Found JustWatch ID: {jw_id} for {title} with TMDB ID: {tmdb_id}")
                    return jw_id, jw_serie_data

        else:
            logger.debug("Could not find a TMDB ID")

        logger.debug(f"Could not find {title} using TVDB ID: {tvdb_id}")
        return None, None

    def _find_serie(self, serie, jw_providers, tmdb_api_key, fast, exclude):
        # Set the minimal base variables
        sonarr_id = serie["id"]
        title = serie["title"]
        release_year = serie["year"]
        providers = [values["short_name"] for _, values in jw_providers.items()]

        # Set extra payload to narrow down the search if fast is true
        jw_query_payload = {}
        if fast:
            jw_query_payload = {
                "page_size": 3,
                "release_year_from": release_year,
                "release_year_until": release_year,
                "monetization_types": ["flatrate"],
            }
            if exclude:
                jw_query_payload.update({"providers": providers})

        # Check if there is an IMDB ID, otherwise check if TMDB API is reachable to get the TMDB ID of the movie
        imdb_id = serie.get("imdbId", None)
        tvdb_id = serie.get("tvdbId", None)
        logger.debug(f"{title} has IMDB ID: {imdb_id} and TVDB_ID: {tvdb_id}")

        # Set JustWatch return variables to None
        jw_id = None
        jw_serie_data = None

        # Setup TMDB if there is an API key provided
        # TODO: set to init
        if tmdb_api_key:
            self.tmdb = pytmdb.TMDB(tmdb_api_key)

        if imdb_id:
            # Try extracting the data by using the IMDB ID
            jw_id, jw_serie_data = self._find_using_imdb_id(
                title, sonarr_id, imdb_id, fast, jw_query_payload
            )
            if not jw_serie_data and tvdb_id and tmdb_api_key:
                logger.debug(f"Could not find {title} using IMDB, falling back to TMDB")
                jw_id, jw_serie_data = self._find_using_tvdb_id(title, sonarr_id, tvdb_id, fast)
        elif tvdb_id and tmdb_api_key:
            # If the user has filled in an TMDB ID fall back to querying TMDB API using the TVDB ID
            jw_id, jw_serie_data = self._find_using_tvdb_id(
                title, sonarr_id, tvdb_id, fast, jw_query_payload
            )
        else:
            # Skip this serie if no IMDB ID and TVDB ID are found
            logger.debug(
                f"No IMDB ID provided by Sonarr and no TMDB configuration set. Skipping serie: {title}"
            )

        return jw_id, jw_serie_data

    def get_series_to_exclude(
        self, providers, fast=True, disable_progress=False, tmdb_api_key=None
    ):
        exclude_series = {}

        # Get all series listed in Sonarr
        logger.debug("Getting all the series from Sonarr")
        sonarr_series = self.sonarr_client.serie.get_all_series()

        # Get the providers listed for the configured locale from JustWatch
        # and filter it with the given providers. This will ensure only the correct
        # providers are in the dictionary.
        raw_jw_providers = self.justwatch_client.get_providers()
        jw_providers = filters.get_providers(raw_jw_providers, providers)
        logger.debug(
            f"Got the following providers: {', '.join([v['clear_name'] for _, v in jw_providers.items()])}"
        )

        progress = Progress(disable=disable_progress)
        with progress:
            for serie in progress.track(sonarr_series):
                # Set the minimal base variables
                sonarr_id = serie["id"]
                title = serie["title"]
                filesize = serie.get("statistics", {}).get("sizeOnDisk", 0)
                release_year = serie["year"]
                ended = serie["ended"]

                # Get episodes of the serie
                episodes = self.sonarr_client.episode.get_episodes_of_serie(sonarr_id)

                # Get JustWatch serie data
                jw_id, jw_serie_data = self._find_serie(
                    serie, jw_providers, tmdb_api_key, fast, exclude=True
                )

                # Continue if the proper JustWatch ID is found
                if jw_serie_data:
                    logger.debug(f"Look up season data for {title}")
                    jw_seasons = jw_serie_data["seasons"]

                    # Loop over the seasons
                    for jw_season in jw_seasons:
                        jw_season_title = jw_season.get(
                            "title", f"Season {jw_season['season_number']}"
                        )
                        jw_season_id = jw_season["id"]
                        jw_season_data = self.justwatch_client.get_season(jw_season_id)
                        jw_episodes = jw_season_data.get("episodes", [])

                        logger.debug(f"Processing season {jw_season_title} of {title}")

                        # Loop over the episodes and check if there are providers
                        for episode in jw_episodes:
                            season_number = episode["season_number"]
                            episode_number = episode["episode_number"]

                            # Get episode providers
                            episode_providers = filters.get_jw_providers(episode)

                            # Check if the providers of the episodes matches the configured providers
                            providers_match = [
                                provider_details["clear_name"]
                                for provider_id, provider_details in jw_providers.items()
                                if provider_id in episode_providers.keys()
                            ]

                            if providers_match:
                                # Get the episode data from the information in Sonarr
                                sonarr_episode_data = filters.get_episode_data(
                                    episodes, season_number, episode_number
                                )
                                sonarr_episode_id = filters.get_episode_file_id(
                                    episodes, season_number, episode_number
                                )

                                exclude_series.update(
                                    {
                                        sonarr_id: {
                                            "title": title,
                                            "filesize": filesize,
                                            "release_year": release_year,
                                            "ended": ended,
                                            "jw_id": jw_id,
                                            "sonarr_object": serie,
                                            "sonarr_file_ids": exclude_series[sonarr_id][
                                                "sonarr_file_ids"
                                            ]
                                            + sonarr_episode_id
                                            if exclude_series.get(sonarr_id)
                                            else sonarr_episode_id,
                                            "episodes": exclude_series[sonarr_id]["episodes"]
                                            + [
                                                {
                                                    "season": season_number,
                                                    "episode": episode_number,
                                                    "providers": providers_match,
                                                    **sonarr_episode_data,
                                                }
                                            ]
                                            if exclude_series.get(sonarr_id)
                                            else [
                                                {
                                                    "season": season_number,
                                                    "episode": episode_number,
                                                    "providers": providers_match,
                                                    **sonarr_episode_data,
                                                }
                                            ],
                                        }
                                    }
                                )

                                logger.debug(
                                    f"{title} S{season_number}E{episode_number} is streaming on {', '.join(providers_match)}"
                                )

        # Check if the full season could be excluded rather than seperate episodes
        for exclude_id, exclude_entry in exclude_series.items():
            sonarr_object = exclude_entry["sonarr_object"]
            sonarr_seasons = sonarr_object["seasons"]
            exclude_episodes = exclude_entry["episodes"]

            seasons_to_exclude = []
            season_numbers = []

            # Loop over the seasons registerd in Sonarr
            for season in sonarr_seasons:
                sonarr_total_episodes = season["statistics"]["totalEpisodeCount"]
                sonarr_season_monitored = season["monitored"]
                sonarr_season_has_file = bool(season["statistics"]["episodeFileCount"])
                sonarr_season_number = int(season["seasonNumber"])

                # Get the total amount of episodes
                exclude_total_episodes = [
                    episode
                    for episode in exclude_episodes
                    if episode["season"] == sonarr_season_number
                ]

                # Get a list of providers of the season
                season_providers = [
                    episode["providers"]
                    for episode in exclude_episodes
                    if episode["season"] == sonarr_season_number
                ]
                season_providers = filters.flatten(season_providers)

                # Check if the amount of episodes to exclude is greater or equal the total episodes in Sonarr
                if len(exclude_total_episodes) >= sonarr_total_episodes:
                    season_numbers.append(sonarr_season_number)
                    seasons_to_exclude.append(
                        {
                            "season": sonarr_season_number,
                            "providers": season_providers,
                            "monitored": sonarr_season_monitored,
                            "has_file": sonarr_season_has_file,
                        }
                    )

            # Re order the exclude_series dict to strip the episodes if the whole season can be excluded
            updated_exclude_episodes = [
                episode for episode in exclude_episodes if episode["season"] not in season_numbers
            ]
            exclude_series[exclude_id]["episodes"] = updated_exclude_episodes
            exclude_series[exclude_id]["seasons"] = seasons_to_exclude
            exclude_series[exclude_id]["providers"] = filters.get_providers_from_seasons_episodes(
                exclude_entry["seasons"], exclude_entry["episodes"]
            )

        return exclude_series

    def get_series_to_re_add(self, providers, fast=True, disable_progress=False, tmdb_api_key=None):
        re_add_series = {}

        # Setup TMDB if there is an API key provided
        if tmdb_api_key:
            self.tmdb = pytmdb.TMDB(tmdb_api_key)

        # Get all series listed in Sonarr
        logger.debug("Getting all the series from Sonarr")
        sonarr_series = self.sonarr_client.serie.get_all_series()

        # Get the providers listed for the configured locale from JustWatch
        # and filter it with the given providers. This will ensure only the correct
        # providers are in the dictionary.
        raw_jw_providers = self.justwatch_client.get_providers()
        jw_providers = filters.get_providers(raw_jw_providers, providers)
        logger.debug(
            f"Got the following providers: {', '.join([v['clear_name'] for _, v in jw_providers.items()])}"
        )

        progress = Progress(disable=disable_progress)
        with progress:
            for serie in progress.track(sonarr_series):
                # Set the minimal base variables
                sonarr_id = serie["id"]
                title = serie["title"]
                release_year = serie["year"]
                ended = serie["ended"]

                # Get episodes of the serie
                episodes = self.sonarr_client.episode.get_episodes_of_serie(sonarr_id)

                # Get JustWatch serie data
                jw_id, jw_serie_data = self._find_serie(
                    serie, jw_providers, tmdb_api_key, fast, exclude=False
                )

                # Continue if the proper JustWatch ID is found
                if jw_serie_data:
                    logger.debug(f"Look up season data for {title}")
                    jw_seasons = jw_serie_data["seasons"]

                    # Loop over the seasons
                    for jw_season in jw_seasons:
                        jw_season_title = jw_season.get(
                            "title", f"Season {jw_season['season_number']}"
                        )
                        jw_season_id = jw_season["id"]
                        jw_season_data = self.justwatch_client.get_season(jw_season_id)
                        jw_episodes = jw_season_data.get("episodes", [])

                        logger.debug(f"Processing season {jw_season_title} of {title}")

                        # Loop over the episodes and check if there are providers
                        for episode in jw_episodes:
                            season_number = episode["season_number"]
                            episode_number = episode["episode_number"]

                            # Get episode providers
                            episode_providers = filters.get_jw_providers(episode)

                            # Check if the providers of the episodes matches the configured providers
                            providers_match = [
                                provider_details["clear_name"]
                                for provider_id, provider_details in jw_providers.items()
                                if provider_id in episode_providers.keys()
                            ]

                            if not providers_match:
                                # Get the episode data from the information in Sonarr
                                sonarr_episode_data = filters.get_episode_data(
                                    episodes, season_number, episode_number
                                )

                                re_add_series.update(
                                    {
                                        sonarr_id: {
                                            "title": title,
                                            "release_year": release_year,
                                            "ended": ended,
                                            "jw_id": jw_id,
                                            "sonarr_object": serie,
                                            "episodes": re_add_series[sonarr_id]["episodes"]
                                            + [
                                                {
                                                    "season": season_number,
                                                    "episode": episode_number,
                                                    **sonarr_episode_data,
                                                }
                                            ]
                                            if re_add_series.get(sonarr_id)
                                            else [
                                                {
                                                    "season": season_number,
                                                    "episode": episode_number,
                                                    **sonarr_episode_data,
                                                }
                                            ],
                                        }
                                    }
                                )

                                logger.debug(
                                    f"{title} S{season_number}E{episode_number} is not streaming on a configured provider"
                                )

        # Check if the full season could be excluded rather than seperate episodes
        for re_add_id, re_add_entry in re_add_series.items():
            sonarr_object = re_add_entry["sonarr_object"]
            sonarr_seasons = sonarr_object["seasons"]
            re_add_episodes = re_add_entry["episodes"]

            seasons_to_re_add = []
            season_numbers = []

            # Loop over the seasons registerd in Sonarr
            for season in sonarr_seasons:
                sonarr_total_episodes = season["statistics"]["totalEpisodeCount"]
                sonarr_season_monitored = season["monitored"]
                sonarr_season_number = int(season["seasonNumber"])

                # Get the total amount of episodes
                re_add_total_episodes = [
                    episode
                    for episode in re_add_episodes
                    if episode["season"] == sonarr_season_number
                ]

                # Check if the amount of episodes to exclude is greater or equals the total episodes in Sonarr
                if len(re_add_total_episodes) >= sonarr_total_episodes:
                    season_numbers.append(sonarr_season_number)
                    seasons_to_re_add.append(
                        {
                            "season": sonarr_season_number,
                            "monitored": sonarr_season_monitored,
                        }
                    )

            # Re order the re_add_series dict to strip the episodes if the whole season can be excluded
            # or if the episode is not monitored but the season is
            updated_re_add_episodes = []
            for episode in re_add_episodes:
                episode_season = episode["season"]
                episode_monitored = episode.get("monitored", True)
                season_monitored = [
                    season["monitored"]
                    for season in seasons_to_re_add
                    if season["season"] == episode_season
                ]

                if all(season_monitored) and not episode_monitored:
                    updated_re_add_episodes.append(episode)
                elif episode_season not in season_numbers:
                    updated_re_add_episodes.append(episode)

            # Save all episode IDs in case we need to re add the whole serie
            all_episode_ids = [
                episode["episode_id"]
                for episode in re_add_episodes
                if episode.get("episode_id", False)
            ]

            re_add_series[re_add_id]["all_episode_ids"] = all_episode_ids
            re_add_series[re_add_id]["episodes"] = updated_re_add_episodes
            re_add_series[re_add_id]["seasons"] = seasons_to_re_add

        return re_add_series

    def delete_serie(self, id, delete_files, add_import_exclusion):
        logger.debug("Starting the delete serie process")

        try:
            logger.debug(f"Deleting serie with Sonarr ID: {id}")
            self.sonarr_client.serie.delete_serie(
                id, delete_files=delete_files, add_import_exclusion=add_import_exclusion
            )
        except Exception as e:
            logger.error(e)
            logger.error(
                f"Something went wrong with deleting the serie from Sonarr, check the configuration or try --debug for more information"
            )

    def delete_episode_files(self, id, episode_file_ids):
        logger.debug(f"Starting the delete episodefile process")

        for episode_file in episode_file_ids:
            try:
                logger.debug(f"Deleting episode ID: {episode_file} for serie with Sonarr ID: {id}")
                self.sonarr_client.episodefile.delete_episodefile(episode_file)
            except Exception as e:
                logger.error(e)
                logger.error(
                    f"Something went wrong with deleting the episodefile with ID: {episode_file}, check the configuration or try --debug for more information"
                )

    def disable_monitored_serie(self, id, sonarr_object):
        logger.debug("Starting to disable monitoring on serie")

        sonarr_object["monitored"] = False

        try:
            logger.debug(f"Updating serie with Sonarr ID: {id}")
            self.sonarr_client.serie.update_serie(sonarr_object)
        except Exception as e:
            logger.error(e)
            logger.error(
                f"Something went wrong with updating the serie with ID: {id} in Sonarr, check the configuration or try --debug for more information"
            )

    def disable_monitored_seasons(self, id, sonarr_object, seasons):
        logger.debug("Starting to disable monitoring on seasons")

        updated_sonarr_object = filters.modify_sonarr_seasons(sonarr_object, seasons, False)

        try:
            logger.debug(f"Updating serie with Sonarr ID: {id}")
            self.sonarr_client.serie.update_serie(updated_sonarr_object)
        except Exception as e:
            logger.error(e)
            logger.error(
                f"Something went wrong with updating the serie with ID: {id} in Sonarr, check the configuration or try --debug for more information"
            )

    def disable_monitored_episodes(self, id, episode_ids):
        logger.debug("Starting to disable monitoring on episodes")

        for episode_id in episode_ids:
            try:
                logger.debug(
                    f"Get details of episode with ID: {episode_id} for serie with Sonarr ID: {id}"
                )
                episode_object = self.sonarr_client.episode.get_episode(episode_id)
                episode_object["monitored"] = False

                logger.debug(
                    f"Updating episode with ID: {episode_id} for serie with Sonarr ID: {id}"
                )
                self.sonarr_client.episode.update_episode(episode_id, episode_object)
            except Exception as e:
                logger.error(e)
                logger.error(
                    f"Something went wrong with updating the episode with ID: {episode_id} in Sonarr, check the configuration or try --debug for more information"
                )

    def enable_monitored_serie(self, id, sonarr_object):
        logger.debug("Starting to enable monitoring on serie")

        sonarr_object["monitored"] = True

        try:
            logger.debug(f"Updating serie with Sonarr ID: {id}")
            self.sonarr_client.serie.update_serie(sonarr_object)
        except Exception as e:
            logger.error(e)
            logger.error(
                f"Something went wrong with updating the serie with ID: {id} in Sonarr, check the configuration or try --debug for more information"
            )

    def enable_monitored_seasons(self, id, sonarr_object, seasons):
        logger.debug("Starting to enable monitoring on seasons")

        updated_sonarr_object = filters.modify_sonarr_seasons(sonarr_object, seasons, True)

        try:
            logger.debug(f"Updating serie with Sonarr ID: {id}")
            self.sonarr_client.serie.update_serie(updated_sonarr_object)
        except Exception as e:
            logger.error(e)
            logger.error(
                f"Something went wrong with updating the serie with ID: {id} in Sonarr, check the configuration or try --debug for more information"
            )

    def enable_monitored_episodes(self, id, episode_ids):
        logger.debug("Starting to enable monitoring on episodes")

        for episode_id in episode_ids:
            try:
                logger.debug(
                    f"Get details of episode with ID: {episode_id} for serie with Sonarr ID: {id}"
                )
                episode_object = self.sonarr_client.episode.get_episode(episode_id)
                episode_object["monitored"] = True

                logger.debug(
                    f"Updating episode with ID: {episode_id} for serie with Sonarr ID: {id}"
                )
                self.sonarr_client.episode.update_episode(episode_id, episode_object)
            except Exception as e:
                logger.error(e)
                logger.error(
                    f"Something went wrong with updating the episode with ID: {episode_id} in Sonarr, check the configuration or try --debug for more information"
                )



import copy

from io import IOBase
from yaml import dump, safe_load
from loguru import logger
from pathlib import Path

from excludarr.utils.redact import redact_config_dict


class Config:
    def __init__(self):
        self.__class__ = Config
        self.config = None

        possible_locations = (
            "/etc/excludarr/excludarr.yml",
            f"{Path.home()}/.config/excludarr/excludarr.yml",
            f"{Path.home()}/.excludarr/config/excludarr.yml",
            f"{Path.home()}/.excludarr.yml",
            "./.excludarr.yml",
        )

        config_file = self.determine_location(possible_locations)

        if config_file is not None:
            self.load(config_file)

    def determine_location(self, possible_locations):
        logger.debug("Determining which configfile to use")

        config_file = None
        for location in possible_locations:
            path = Path(location)

            if path.is_file():
                config_file = location

        logger.debug(f"Configfile to use: {config_file}")

        return config_file

    def load(self, config_file):
        logger.debug(f"Reading configfile: {config_file}")

        if isinstance(config_file, IOBase):
            self.config = safe_load(config_file)
        else:
            with open(config_file, "r") as _file:
                self.config = safe_load(_file)

        logger.debug(
            f"Read the following configuration: {redact_config_dict(copy.deepcopy(self.config))}"
        )

    def dump(self):
        return dump(self.config)

    @property
    def general_section(self):
        return self.config.get("general", {})

    @property
    def tmdb_section(self):
        return self.config.get("tmdb", {})

    @property
    def radarr_section(self):
        return self.config.get("radarr", {})

    @property
    def sonarr_section(self):
        return self.config.get("sonarr", {})

    @property
    def locale(self):
        return self.general_section.get("locale", None)

    @property
    def providers(self):
        return self.general_section.get("providers", [])

    @property
    def fast_search(self):
        return self.general_section.get("fast_search", True)

    @property
    def tmdb_api_key(self):
        return self.tmdb_section.get("api_key", None)

    @property
    def radarr_url(self):
        return self.radarr_section.get("url", None)

    @property
    def radarr_api_key(self):
        return self.radarr_section.get("api_key", None)

    @property
    def radarr_verify_ssl(self):
        return self.radarr_section.get("verify_ssl", False)

    @property
    def radarr_excludes(self):
        return self.radarr_section.get("exclude", [])

    @property
    def sonarr_url(self):
        return self.sonarr_section.get("url", None)

    @property
    def sonarr_api_key(self):
        return self.sonarr_section.get("api_key", None)

    @property
    def sonarr_verify_ssl(self):
        return self.sonarr_section.get("verify_ssl", False)

    @property
    def sonarr_excludes(self):
        return self.sonarr_section.get("exclude", [])



import typer
import sys

from typing import Optional
from loguru import logger

import excludarr.commands.radarr as radarr
import excludarr.commands.sonarr as sonarr
import excludarr.commands.providers as providers

from excludarr import __version__


app = typer.Typer()
app.add_typer(radarr.app, name="radarr", help="Manages movies in Radarr.")
app.add_typer(sonarr.app, name="sonarr", help="Manages TV shows, seasons and episodes in Sonarr.")
app.add_typer(
    providers.app, name="providers", help="List all the possible providers for your locale."
)


def version_callback(value: bool):
    if value:
        typer.echo(f"Excludarr: v{__version__}")
        raise typer.Exit()


def _setup_logging(debug):
    """
    Setup the log formatter for Excludarr
    """

    log_level = "INFO"
    if debug:
        log_level = "DEBUG"

    logger.remove()
    logger.add(
        sys.stdout,
        colorize=True,
        format="[{time:YYYY-MM-DD HH:mm:ss}] - <level>{message}</level>",
        level=log_level,
    )


@app.callback()
def main(
    debug: bool = False,
    version: Optional[bool] = typer.Option(None, "--version", callback=version_callback),
):
    """
    Excludarr is a CLI that interacts with Radarr and Sonarr instances. It completely
    manages you library in Sonarr and Radarr to only consist out of movies and series that
    are not present on any of the configured streaming providers. Excludarr can also
    re monitor movies and series if it is not available anymore on any of the configured
    streaming providers. You can also configure to delete the already downloaded files of
    the excluded entry to keep your storage happy!
    """

    # Setup the logger
    _setup_logging(debug)

    # Logging
    logger.debug(f"Starting Excludarr v{__version__}")


def cli():
    app(prog_name="excludarr")


if __name__ == "__main__":
    cli()



class SonarrTooManyVariablesException(Exception):
    pass


class SonarrInvalidIdSupplied(Exception):
    pass


class SonarrInvalidApiKey(Exception):
    pass


class SonarrMovieNotFound(Exception):
    pass


class SonarrValidationException(Exception):
    pass



def redact_config_dict(data):
    """
    This function redacts secret values in the configuration dict.
    This prevents that logging prints plain api keys.
    """
    for key, value in data.items():
        # Redact the secret values
        if key == "api_key" and data[key]:
            data[key] = "<REDACTED>"

        # Check if there are still more dicts to loop over
        if isinstance(value, dict):
            redact_config_dict(value)

    return data



from ..base import Manager


class Movie(Manager):
    def get_details(self, movie_id):
        path = "/movie/{}".format(movie_id)

        return self.client.http_get(path)

    def get_watch_providers(self, movie_id):
        path = "/movie/{}/watch/providers".format(movie_id)

        return self.client.http_get(path)



from platform import release
from loguru import logger
from rich.progress import Progress

import excludarr.modules.pyradarr as pyradarr
import excludarr.utils.filters as filters

from excludarr.modules.justwatch import JustWatch
from excludarr.modules.justwatch.exceptions import JustWatchNotFound, JustWatchTooManyRequests
from excludarr.modules.pyradarr.exceptions import RadarrMovieNotFound


class RadarrActions:
    def __init__(self, url, api_key, locale, verify_ssl=False):
        logger.debug(f"Initializing PyRadarr")
        self.radarr_client = pyradarr.Radarr(url, api_key, verify_ssl)

        logger.debug(f"Initializing JustWatch API with locale: {locale}")
        self.justwatch_client = JustWatch(locale)

    def _get_jw_movie_data(self, title, jw_entry):
        jw_id = jw_entry["id"]
        jw_movie_data = {}
        jw_tmdb_ids = []

        try:
            logger.debug(f"Querying JustWatch API with ID: {jw_id} for title: {title}")
            jw_movie_data = self.justwatch_client.get_movie(jw_id)

            jw_tmdb_ids = filters.get_tmdb_ids(jw_movie_data.get("external_ids", []))
            logger.debug(f"Got TMDB ID's: {jw_tmdb_ids} from JustWatch API")
        except JustWatchNotFound:
            logger.warning(f"Could not find title: {title} with JustWatch ID: {jw_id}")
        except JustWatchTooManyRequests:
            logger.error(f"JustWatch API returned 'Too Many Requests'")
            # TODO: Raise error so typer can abort properly

        return jw_movie_data, jw_tmdb_ids

    def _find_movie(self, movie, jw_providers, fast, exclude):
        # Set the minimal base variables
        title = movie["title"]
        tmdb_id = movie["tmdbId"]
        release_year = filters.get_release_date(movie, format="%Y")
        providers = [values["short_name"] for _, values in jw_providers.items()]

        # Set extra payload to narrow down the search if fast is true
        jw_query_payload = {}
        if fast:
            jw_query_payload.update({"page_size": 3})

            if exclude:
                jw_query_payload.update(
                    {"monetization_types": ["flatrate"], "providers": providers}
                )

            if release_year:
                jw_query_payload.update(
                    {
                        "release_year_from": int(release_year),
                        "release_year_until": int(release_year),
                    }
                )

        # Log the JustWatch API call function
        logger.debug(f"Query JustWatch API with title: {title}")
        jw_query_data = self.justwatch_client.query_title(title, "movie", fast, **jw_query_payload)

        for entry in jw_query_data["items"]:
            jw_id = entry["id"]
            jw_movie_data, jw_tmdb_ids = self._get_jw_movie_data(title, entry)

            # Break if the TMBD_ID in the query of JustWatch matches the one in Radarr
            if tmdb_id in jw_tmdb_ids:
                logger.debug(f"Found JustWatch ID: {jw_id} for {title} with TMDB ID: {tmdb_id}")
                return jw_id, jw_movie_data

        return None, None

    def get_movies_to_exclude(self, providers, fast=True, disable_progress=False):
        exclude_movies = {}

        # Get all movies listed in Radarr
        logger.debug("Getting all the movies from Radarr")
        radarr_movies = self.radarr_client.movie.get_all_movies()

        # Get the providers listed for the configured locale from JustWatch
        # and filter it with the given providers. This will ensure only the correct
        # providers are in the dictionary.
        raw_jw_providers = self.justwatch_client.get_providers()
        jw_providers = filters.get_providers(raw_jw_providers, providers)
        logger.debug(
            f"Got the following providers: {', '.join([v['clear_name'] for _, v in jw_providers.items()])}"
        )

        progress = Progress(disable=disable_progress)
        with progress:
            for movie in progress.track(radarr_movies):
                # Set the minimal base variables
                radarr_id = movie["id"]
                title = movie["title"]
                tmdb_id = movie["tmdbId"]
                filesize = movie["sizeOnDisk"]
                release_date = filters.get_release_date(movie)

                # Log the title and Radarr ID
                logger.debug(
                    f"Processing title: {title} with Radarr ID: {radarr_id} and TMDB ID: {tmdb_id}"
                )

                # Find the movie
                jw_id, jw_movie_data = self._find_movie(movie, jw_providers, fast, exclude=True)

                if jw_movie_data:
                    # Get all the providers the movie is streaming on
                    movie_providers = filters.get_jw_providers(jw_movie_data)

                    # Loop over the configured providers and check if the provider
                    # matches the providers advertised at the movie. If a match is found
                    # update the exclude_movies dict
                    matched_providers = list(set(movie_providers.keys()) & set(jw_providers.keys()))

                    if matched_providers:
                        clear_names = [
                            provider_details["clear_name"]
                            for provider_id, provider_details in jw_providers.items()
                            if provider_id in matched_providers
                        ]

                        exclude_movies.update(
                            {
                                radarr_id: {
                                    "title": title,
                                    "filesize": filesize,
                                    "release_date": release_date,
                                    "radarr_object": movie,
                                    "tmdb_id": tmdb_id,
                                    "jw_id": jw_id,
                                    "providers": clear_names,
                                }
                            }
                        )

                        logger.debug(f"{title} is streaming on {', '.join(clear_names)}")

        return exclude_movies

    def get_movies_to_re_add(self, providers, fast=True, disable_progress=False):
        re_add_movies = {}

        # Get all movies listed in Radarr and filter it to only include not monitored movies
        logger.debug("Getting all the movies from Radarr")
        radarr_movies = self.radarr_client.movie.get_all_movies()
        radarr_not_monitored_movies = [movie for movie in radarr_movies if not movie["monitored"]]

        # Get the providers listed for the configured locale from JustWatch
        # and filter it with the given providers. This will ensure only the correct
        # providers are in the dictionary.
        raw_jw_providers = self.justwatch_client.get_providers()
        jw_providers = filters.get_providers(raw_jw_providers, providers)
        logger.debug(
            f"Got the following providers: {', '.join([v['clear_name'] for _, v in jw_providers.items()])}"
        )

        progress = Progress(disable=disable_progress)
        with progress:
            for movie in progress.track(radarr_not_monitored_movies):
                # Set the minimal base variables
                radarr_id = movie["id"]
                title = movie["title"]
                tmdb_id = movie["tmdbId"]
                release_date = filters.get_release_date(movie)

                # Log the title and Radarr ID
                logger.debug(
                    f"Processing title: {title} with Radarr ID: {radarr_id} and TMDB ID: {tmdb_id}"
                )

                # Find the movie
                jw_id, jw_movie_data = self._find_movie(movie, jw_providers, fast, exclude=False)

                if jw_movie_data:
                    # Get all the providers the movie is streaming on
                    movie_providers = filters.get_jw_providers(jw_movie_data)

                    # Check if the JustWatch providers matching the movie providers
                    matched_providers = list(set(movie_providers.keys()) & set(jw_providers.keys()))

                    if not matched_providers:
                        re_add_movies.update(
                            {
                                radarr_id: {
                                    "title": title,
                                    "release_date": release_date,
                                    "radarr_object": movie,
                                    "tmdb_id": tmdb_id,
                                    "jw_id": jw_id,
                                }
                            }
                        )
                        logger.debug(f"{title} is not streaming on a configured provider")

        return re_add_movies

    def delete(self, ids, delete_files, add_import_exclusion):
        logger.debug("Starting the delete process")

        try:
            logger.debug("Trying to bulk delete all movies at once")

            self.radarr_client.movie.delete_movies(
                ids, delete_files=delete_files, add_import_exclusion=add_import_exclusion
            )
        except RadarrMovieNotFound:
            logger.warning("Bulk delete failed, falling back to deleting each movie individually")
            for id in ids:
                logger.debug(f"Deleting movie with Radarr ID: {id}")

                self.radarr_client.movie.delete_movie(
                    id, delete_files=delete_files, add_import_exclusion=add_import_exclusion
                )
        except Exception as e:
            logger.error(e)
            logger.error(
                f"Something went wrong with deleting the movies from Radarr, check the configuration or try --debug for more information"
            )

    def disable_monitored(self, movies):
        logger.debug("Starting the process of changing the status to not monitored")
        for movie in movies:
            movie.update({"monitored": False})

            logger.debug(f"Change monitored to False for movie with Radarr ID: {movie['id']}")
            self.radarr_client.movie.update_movie(movie)

    def enable_monitored(self, movies):
        logger.debug("Starting the process of changing the status to monitored")
        for movie in movies:
            movie.update({"monitored": True})

            logger.debug(f"Change monitored to True for movie with Radarr ID: {movie['id']}")
            self.radarr_client.movie.update_movie(movie)

    def delete_files(self, ids):
        logger.debug("Starting the process of deleting the files")
        for id in ids:
            logger.debug(f"Checking if movie with Radarr ID: {id} has files")
            moviefiles = self.radarr_client.moviefile.get_moviefiles(id)

            for moviefile in moviefiles:
                logger.debug(f"Deleting files for movie with Radarr ID: {id}")
                self.radarr_client.moviefile.delete_moviefile(moviefile["id"])



class TMDBException(Exception):
    def __init__(self, error_code, error_message):
        Exception.__init__(self, error_message)
        self.error_code = error_code
        self.error_message = error_message

    def __str__(self):
        return "[{}] - {}".format(self.error_code, self.error_message)



from ..base import Manager


class Episode(Manager):
    def __init__(self, client):
        super().__init__(client)
        self.episode_path = "/episode"
        self.episode_id_path = "/episode/{id}".format(id="{id}")

    def get_episodes_of_serie(self, series_id):
        params = {
            "seriesId": series_id,
        }
        return self.client.http_get(self.episode_path, params=params)

    def get_episode(self, id):
        return self.client.http_get(self.episode_id_path.format(id=id))

    def update_episode(self, id, updated_episode_dict):
        return self.client.http_put(self.episode_id_path.format(id=id), json=updated_episode_dict)



import rich

from rich.console import Console
from rich.table import Table
from rich.live import Live
from rich.prompt import Confirm
from rich.text import Text
from rich import box

import excludarr.utils.filters as filters

from excludarr.utils.enums import Action


def print_movies_to_exclude(movies, total_filesize):
    # Setup console
    console = Console()

    # Setup table
    table = Table(show_footer=True, row_styles=["none", "dim"], box=box.MINIMAL, pad_edge=False)
    with Live(table, console=console, screen=False):
        # Setup table columns and totals
        table.add_column("Release Date") or "Unknown"
        table.add_column("Title", Text.from_markup("[b][i]Total Used Diskspace", justify="right"))
        table.add_column("Used Diskspace", filters.get_filesize_gb(total_filesize))
        table.add_column("Streaming Providers")

        for _, movie in movies.items():
            release_date = movie["release_date"]
            title = movie["title"]
            diskspace = filters.get_filesize_gb(movie["filesize"])
            providers = ", ".join(movie["providers"])

            # Add table rows
            table.add_row(release_date, title, diskspace, providers)


def print_movies_to_re_add(movies):
    # Setup console
    console = Console()

    # Setup table
    table = Table(show_footer=False, row_styles=["none", "dim"], box=box.MINIMAL, pad_edge=False)
    with Live(table, console=console, screen=False):
        # Setup table columns
        table.add_column("Release Date")
        table.add_column("Title")

        for _, movie in movies.items():
            release_date = movie["release_date"]
            title = movie["title"]

            # Add table rows
            table.add_row(release_date, title)


def print_series_to_exclude(series, total_filesize):
    # Setup console
    console = Console()

    # Setup table
    table = Table(show_footer=True, row_styles=["none", "dim"], box=box.MINIMAL, pad_edge=False)
    with Live(table, console=console, screen=False):
        # Setup table columns and totals
        table.add_column("Release Year")
        table.add_column("Title", Text.from_markup("[b][i]Total Used Diskspace", justify="right"))
        table.add_column("Used Diskspace", filters.get_filesize_gb(total_filesize))
        table.add_column("Seasons")
        table.add_column("Episodes")
        table.add_column("Providers")
        table.add_column("Ended")
        table.add_column("Full delete")

        for _, serie in series.items():
            release_year = str(serie["release_year"])
            title = serie["title"]
            diskspace = filters.get_filesize_gb(serie["filesize"])
            season = filters.get_pretty_seasons(serie["seasons"])
            episodes = filters.get_pretty_episodes(serie["episodes"])
            providers = serie["providers"]
            ended = filters.bool2str(serie["ended"])
            full_delete = filters.bool2str(serie["full_delete"])

            # Add table rows
            table.add_row(
                release_year, title, diskspace, season, episodes, providers, ended, full_delete
            )


def print_series_to_re_add(series):
    # Setup console
    console = Console()

    # Setup table
    table = Table(show_footer=False, row_styles=["none", "dim"], box=box.MINIMAL, pad_edge=False)
    with Live(table, console=console, screen=False):
        # Setup table columns
        table.add_column("Release Year")
        table.add_column("Title")
        table.add_column("Seasons")
        table.add_column("Episodes")
        table.add_column("Ended")

        for _, serie in series.items():
            release_year = str(serie["release_year"])
            title = serie["title"]
            season = filters.get_pretty_seasons(serie["seasons"])
            episodes = filters.get_pretty_episodes(serie["episodes"])
            ended = filters.bool2str(serie["ended"])

            # Add table rows
            table.add_row(release_year, title, season, episodes, ended)


def print_providers(providers):
    # Setup console
    console = Console()

    # Setup table
    table = Table(show_footer=False, row_styles=["none", "dim"], box=box.MINIMAL, pad_edge=False)
    with Live(table, console=console, screen=False):
        # Setup table columns
        table.add_column("JustWatch ID")
        table.add_column("Provider")

        for provider in providers:
            id = str(provider["id"])
            clear_name = provider["clear_name"]

            # Add table rows
            table.add_row(id, clear_name)


def ask_confirmation(action, kind):
    if action == Action.delete:
        confirmation = Confirm.ask(
            f"Are you sure you want to delete the listed {kind}?", default=False
        )
    elif action == Action.not_monitored:
        confirmation = Confirm.ask(
            f"Are you sure you want to change the listed {kind} to not-monitored?", default=False
        )
    elif action == "re-add":
        confirmation = Confirm.ask(
            f"Are you sure you want to re monitor the listed {kind}?", default=False
        )

    return confirmation


def print_success_exclude(action, kind):
    if action == Action.delete and kind == "series":
        rich.print(
            "Succesfully deleted the series and/or changed the status of serveral seasons and episodes listed in Sonarr to not monitored!"
        )
    elif action == Action.delete and kind == "movies":
        rich.print("Succesfully deleted the movies from Radarr!")
    elif action == Action.not_monitored and kind == "series":
        rich.print(
            "Succesfully changed the status of the series and/or several seasons and episodes listed in Sonarr to not monitored!"
        )
    elif action == Action.not_monitored and kind == "movies":
        rich.print(
            "Succesfully changed the status of the movies listed in Radarr to not monitored!"
        )



import rich
import typer

from typing import List, Optional
from loguru import logger

import excludarr.utils.output as output

from excludarr.core.sonarr_actions import SonarrActions
from excludarr.utils.config import Config
from excludarr.utils.enums import Action

app = typer.Typer()


@app.command(help="Exclude TV shows in Sonarr by deleting or not monitoring them")
def exclude(
    providers: Optional[List[str]] = typer.Option(
        None,
        "-p",
        "--provider",
        metavar="PROVIDER",
        help="Override the configured streaming providers.",
    ),
    locale: Optional[str] = typer.Option(
        None, "-l", "--locale", metavar="LOCALE", help="Your locale e.g: en_US."
    ),
    action: Action = typer.Option(..., "-a", "--action", help="Change the status in Radarr."),
    delete_files: bool = typer.Option(
        False, "-d", "--delete-files", help="Delete already downloaded files."
    ),
    exclusion: bool = typer.Option(
        False, "-e", "--exclusion", help="Add an exclusion to prevent auto importing."
    ),
    yes: bool = typer.Option(False, "-y", "--yes", help="Auto accept the confirmation notice."),
    progress: bool = typer.Option(
        False, "--progress", help="Track the progress using a progressbar."
    ),
):
    # Debug logging
    logger.debug("Got exclude as subcommand")
    logger.debug(f"Got CLI values for -p, --provider option: {', '.join(providers)}")
    logger.debug(f"Got CLI values for -l, --locale option: {locale}")
    logger.debug(f"Got CLI values for -a, --action option: {action}")
    logger.debug(f"Got CLI values for -d, --delete option: {delete_files}")
    logger.debug(f"Got CLI values for -e, --exclusion option: {exclusion}")
    logger.debug(f"Got CLI values for -y, --yes option: {yes}")
    logger.debug(f"Got CLI values for --progress option: {progress}")

    # Disable the progress bar when debug logging is active
    if loglevel == 10:
        disable_progress = True
    elif progress and loglevel != 10:
        disable_progress = False
    else:
        disable_progress = True

    # Determine if CLI options should overwrite configuration settings
    if not providers:
        providers = config.providers
    if not locale:
        locale = config.locale

    # Setup Sonarr Actions to control the different tasks
    sonarr = SonarrActions(
        config.sonarr_url, config.sonarr_api_key, locale, config.sonarr_verify_ssl
    )

    series_to_exclude = sonarr.get_series_to_exclude(
        providers, config.fast_search, disable_progress, tmdb_api_key=config.tmdb_api_key
    )

    # Only take monitored seasons and episodes in encounter
    for sonarr_id, values in series_to_exclude.items():
        if delete_files:
            values["episodes"] = [
                episode
                for episode in values["episodes"]
                if episode.get("monitored", False) or episode.get("has_file", False)
            ]
            values["seasons"] = [
                season
                for season in values["seasons"]
                if season.get("monitored", False) or season.get("has_file", False)
            ]
        else:
            values["episodes"] = [
                episode for episode in values["episodes"] if episode.get("monitored", False)
            ]
            values["seasons"] = [
                season for season in values["seasons"] if season.get("monitored", False)
            ]

        # Determine if serie should be deleted fully
        sonarr_total_monitored_seasons = len(
            [
                season
                for season in values["sonarr_object"]["seasons"]
                if season.get("monitored", False)
            ]
        )
        total_seasons = len([season["season"] for season in values["seasons"]])

        if (
            total_seasons == sonarr_total_monitored_seasons
            and values["ended"]
            and action == Action.delete
        ):
            values["full_delete"] = True
        else:
            values["full_delete"] = False

    if action == Action.not_monitored:
        series_to_exclude = {
            id: values
            for id, values in series_to_exclude.items()
            if (values["episodes"] or values["seasons"])
            and values["title"] not in config.sonarr_excludes
        }
    else:
        series_to_exclude = {
            id: values
            for id, values in series_to_exclude.items()
            if (values["episodes"] or values["seasons"])
            or values["full_delete"]
            and values["title"] not in config.sonarr_excludes
        }

    # Create a list of the Sonarr IDs
    series_to_exclude_ids = list(series_to_exclude.keys())

    # If there are series to exclude
    if series_to_exclude_ids:
        # Calculate total filesize
        total_filesize = sum([serie["filesize"] for _, serie in series_to_exclude.items()])

        # Print the serie in table format
        output.print_series_to_exclude(series_to_exclude, total_filesize)

        # Check for confirmation
        if not yes:
            confirmation = output.ask_confirmation(action, "series")
            if not confirmation:
                logger.warning("Aborting Excludarr because user did not confirm the question")
                raise typer.Abort()
        else:
            confirmation = True

        if confirmation:
            for sonarr_id, data in series_to_exclude.items():
                sonarr_object = data["sonarr_object"]
                sonarr_total_seasons = sonarr_object["statistics"]["seasonCount"]
                sonarr_full_delete = data["full_delete"]
                seasons = [season["season"] for season in data["seasons"]]
                episodes = data["episodes"]
                episode_ids = [
                    episode["episode_id"]
                    for episode in episodes
                    if episode.get("episode_id", False)
                ]
                episode_files = data["sonarr_file_ids"]

                # Check if total seasons match with the amount of seasons to exclude and delete or
                # change the status to not monitored for the whole serie
                if sonarr_full_delete:
                    if action == Action.delete:
                        sonarr.delete_serie(sonarr_id, delete_files, exclusion)
                    elif action == Action.not_monitored:
                        sonarr.disable_monitored_serie(sonarr_id, sonarr_object)
                        sonarr.disable_monitored_seasons(
                            sonarr_id, sonarr_object, list(range(sonarr_total_seasons + 1))
                        )

                        # Delete the episode files if delete flag is set
                        if delete_files and episode_files:
                            sonarr.delete_episode_files(sonarr_id, episode_files)
                else:
                    # Set the seasons and episodes to not-monitored
                    if seasons:
                        sonarr.disable_monitored_seasons(sonarr_id, sonarr_object, seasons)
                    if episodes:
                        sonarr.disable_monitored_episodes(sonarr_id, episode_ids)

                    # Delete the episode files if delete flag is set
                    if delete_files and episode_files:
                        sonarr.delete_episode_files(sonarr_id, episode_files)

            output.print_success_exclude(action, "series")
    else:
        rich.print("There are no more series also available on the configured streaming providers!")


@app.command(help="Change status of series to monitored if no provider is found")
def re_add(
    providers: Optional[List[str]] = typer.Option(
        None,
        "-p",
        "--provider",
        metavar="PROVIDER",
        help="Override the configured streaming providers.",
    ),
    locale: Optional[str] = typer.Option(
        None, "-l", "--locale", metavar="LOCALE", help="Your locale e.g: en_US."
    ),
    yes: bool = typer.Option(False, "-y", "--yes", help="Auto accept the confirmation notice."),
    progress: bool = typer.Option(
        False, "--progress", help="Track the progress using a progressbar."
    ),
):
    # Debug logging
    logger.debug("Got exclude as subcommand")
    logger.debug(f"Got CLI values for -p, --provider option: {', '.join(providers)}")
    logger.debug(f"Got CLI values for -l, --locale option: {locale}")
    logger.debug(f"Got CLI values for -y, --yes option: {yes}")
    logger.debug(f"Got CLI values for --progress option: {progress}")

    # Disable the progress bar when debug logging is active
    if loglevel == 10:
        disable_progress = True
    elif progress and loglevel != 10:
        disable_progress = False
    else:
        disable_progress = True

    # Determine if CLI options should overwrite configuration settings
    if not providers:
        providers = config.providers
    if not locale:
        locale = config.locale

    # Setup Sonarr Actions to control the different tasks
    sonarr = SonarrActions(
        config.sonarr_url, config.sonarr_api_key, locale, config.sonarr_verify_ssl
    )

    series_to_re_add = sonarr.get_series_to_re_add(
        providers, config.fast_search, disable_progress, tmdb_api_key=config.tmdb_api_key
    )

    # Only take not monitored seasons and episodes in encounter
    for _, values in series_to_re_add.items():
        values["episodes"] = [
            episode for episode in values["episodes"] if not episode.get("monitored", True)
        ]
        values["seasons"] = [
            season for season in values["seasons"] if not season.get("monitored", True)
        ]

    series_to_re_add = {
        id: values
        for id, values in series_to_re_add.items()
        if (values["episodes"] or values["seasons"] or not values["sonarr_object"]["monitored"])
        and values["title"] not in config.sonarr_excludes
    }

    # Create a list of the Sonarr IDs
    series_to_re_add_ids = list(series_to_re_add.keys())

    # If there are series to exclude
    if series_to_re_add_ids:
        # Print the series in table format
        output.print_series_to_re_add(series_to_re_add)

        # Check for confirmation
        if not yes:
            confirmation = output.ask_confirmation("re-add", "series")
            if not confirmation:
                logger.warning("Aborting Excludarr because user did not confirm the question")
                raise typer.Abort()
        else:
            confirmation = True

        if confirmation:
            for sonarr_id, data in series_to_re_add.items():
                sonarr_object = data["sonarr_object"]
                sonarr_total_seasons = sonarr_object["statistics"]["seasonCount"]
                sonarr_total_not_monitored_seasons = len(
                    [season for season in sonarr_object["seasons"] if not season["monitored"]]
                )
                seasons = [season["season"] for season in data["seasons"]]
                total_seasons = len(seasons)
                episodes = data["episodes"]
                episode_ids = [
                    episode["episode_id"]
                    for episode in episodes
                    if episode.get("episode_id", False)
                ]
                all_episode_ids = data["all_episode_ids"]

                # Check if total seasons match with the amount of seasons to re-add and
                # change the status to monitored for the whole serie, seasons and episodes
                if (
                    sonarr_total_not_monitored_seasons == total_seasons
                    and sonarr_total_not_monitored_seasons != 0
                ):
                    sonarr.enable_monitored_serie(sonarr_id, sonarr_object)
                    sonarr.enable_monitored_seasons(
                        sonarr_id, sonarr_object, list(range(sonarr_total_seasons + 1))
                    )
                    sonarr.enable_monitored_episodes(sonarr_id, all_episode_ids)

                else:
                    # Enable monitoring on the serie object it self
                    sonarr.enable_monitored_serie(sonarr_id, sonarr_object)
                    # Set the seasons and episodes to monitored
                    if seasons:
                        sonarr.enable_monitored_seasons(sonarr_id, sonarr_object, seasons)
                    if episodes:
                        sonarr.enable_monitored_episodes(sonarr_id, episode_ids)

            rich.print(
                "Succesfully changed the status of the series listed in Sonarr to monitored!"
            )
    else:
        rich.print("There are no more series to re-add!")


@app.callback()
def init():
    """
    Initializes the command. Reads the configuration.
    """
    logger.debug("Got sonarr as subcommand")

    # Set globals
    global config
    global loglevel

    # Hacky way to get the current log level context
    loglevel = logger._core.min_level

    logger.debug("Reading configuration file")
    config = Config()


if __name__ == "__main__":
    app()



import requests

from urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter
from json import JSONDecodeError

from .exceptions import JustWatchTooManyRequests, JustWatchNotFound, JustWatchBadRequest


class JustWatch(object):
    def __init__(self, locale, ssl_verify=True):
        # Setup base variables
        self.base_url = "https://apis.justwatch.com/content"
        self.ssl_verify = ssl_verify

        # Setup session
        self.session = requests.Session()
        self.session.verify = ssl_verify

        # Setup retries on failure
        retries = Retry(
            total=5,
            backoff_factor=0.5,
            status_forcelist=[429, 500, 502, 503, 504],
            method_whitelist=["GET", "POST"],
        )

        self.session.mount("http://", HTTPAdapter(max_retries=retries))
        self.session.mount("https://", HTTPAdapter(max_retries=retries))

        # Setup locale by verifying its input
        self.locale = self._get_full_locale(locale)

    def __exit__(self, *args):
        self.session.close()

    def _build_url(self, path):
        return "{}{}".format(self.base_url, path)

    def _filter_api_error(self, data):

        if data.status_code == 400:
            raise JustWatchBadRequest(data.text)
        elif data.status_code == 404:
            raise JustWatchNotFound()
        elif data.status_code == 429:
            raise JustWatchTooManyRequests()

        try:
            result_json = data.json()
        except JSONDecodeError:
            return data.text

        return result_json

    def _http_request(self, method, path, json=None, params=None):
        url = self._build_url(path)
        request = requests.Request(method, url, json=json, params=params)

        prepped = self.session.prepare_request(request)
        result = self.session.send(prepped)

        return self._filter_api_error(result)

    def _http_get(self, path, params=None):
        return self._http_request("get", path, params=params)

    def _http_post(self, path, json=None):
        return self._http_request("post", path, json=json)

    def _http_put(self, path, params=None, json=None):
        return self._http_request("put", path, params=params, json=json)

    def _http_delete(self, path, json=None, params=None):
        return self._http_request("delete", path, json=json, params=params)

    def _get_full_locale(self, locale):
        default_locale = "en_US"
        path = "/locales/state"

        jw_locales = self._http_get(path)

        valid_locale = any([True for i in jw_locales if i["full_locale"] == locale])

        # Check if the locale is a iso_3166_2 Country Code
        if not valid_locale:
            locale = "".join([i["full_locale"] for i in jw_locales if i["iso_3166_2"] == locale])

        # If the locale is empty return the default locale
        if not locale:
            return default_locale

        return locale

    def get_providers(self):
        path = f"/providers/locale/{self.locale}"

        return self._http_get(path)

    def query_title(self, query, content_type, fast=True, result={}, page=1, **kwargs):
        """
        Query JustWatch API to find information about a title

        :query: the title of the show or movie to search for
        :content_type: can either be 'show' or 'movie'. Can also be a list of types.
        """
        path = f"/titles/{self.locale}/popular"

        if isinstance(content_type, str):
            content_type = content_type.split(",")

        json = {"query": query, "content_types": content_type}
        if kwargs:
            json.update(kwargs)

        page_result = self._http_post(path, json=json)
        result.update(page_result)

        if not fast and page < result["total_pages"]:
            page += 1
            self.query_title(query, content_type, fast=fast, result=result, page=page)

        return result

    def get_movie(self, jw_id):
        path = f"/titles/movie/{jw_id}/locale/{self.locale}"

        return self._http_get(path)

    def get_show(self, jw_id):
        path = f"/titles/show/{jw_id}/locale/{self.locale}"

        return self._http_get(path)

    def get_season(self, jw_id):
        path = f"/titles/show_season/{jw_id}/locale/{self.locale}"

        return self._http_get(path)



from ..base import Manager


class Find(Manager):
    def find_by_id(self, external_id, external_source):
        path = "/find/{}".format(external_id)

        params = {"external_source": external_source}
        return self.client.http_get(path, params=params)



#!/usr/bin/env python

from ..base import Manager


class MovieFile(Manager):
    def __init__(self, client):
        super().__init__(client)
        self.moviefile_path = "/moviefile"
        self.moviefile_id_path = "/moviefile/{id}".format(id="{id}")

    def get_moviefiles(self, ids):
        params = {"movieId": ids}
        return self.client.http_get(self.moviefile_path, params=params)

    # ID is the file ID, not the ID of the movie
    def get_moviefile_by_file_id(self, id):
        return self.client.http_get(self.moviefile_id_path.format(id=id))

    # ID is the file ID, not the ID of the movie
    def delete_moviefile(self, id):
        return self.client.http_delete(self.moviefile_id_path.format(id=id))



#!/usr/bin/env python

from ..base import Manager


class EpisodeFile(Manager):
    def __init__(self, client):
        super().__init__(client)
        self.episodefile_path = "/episodefile"
        self.episodefile_id_path = "/episodefile/{id}".format(id="{id}")

    def get_episodefiles(self, series_id):
        params = {
            "seriesId": series_id,
        }
        return self.client.http_get(self.episodefile_path, params=params)

    def get_episodefile(self, id):
        return self.client.http_get(self.episodefile_id_path.format(id=id))

    def delete_episodefile(self, id):
        return self.client.http_delete(self.episodefile_id_path.format(id=id))



import datetime
import itertools


def flatten(lst):
    return list(set(itertools.chain.from_iterable(lst)))


def bool2str(value):
    if value:
        return "Yes"
    else:
        return "No"


def get_tmdb_ids(external_ids):
    try:
        tmdb_ids = [
            int(x["external_id"])
            for x in external_ids
            if (x["provider"] == "tmdb_latest" or x["provider"] == "tmdb") and x["external_id"].isnumeric()
        ]
        tmdb_ids = list(set(tmdb_ids))
    except (KeyError, IndexError):
        tmdb_ids = []

    return tmdb_ids

def get_imdb_ids(external_ids):
    try:
        imdb_ids = [
            x["external_id"]
            for x in external_ids
            if x["provider"] == "imdb_latest" or x["provider"] == "imdb"
        ]
        imdb_ids = list(set(imdb_ids))
    except (KeyError, IndexError):
        imdb_ids = []

    return imdb_ids


def get_providers(raw_providers, providers):
    providers = [x.lower() for x in providers]
    jw_providers = {}

    for entry in raw_providers:
        if entry["clear_name"].lower() in providers:
            jw_providers.update(
                {
                    entry["id"]: {
                        "short_name": entry["short_name"],
                        "clear_name": entry["clear_name"],
                    }
                }
            )

    return jw_providers


def get_jw_providers(raw_data):
    providers = {}

    try:
        for entry in raw_data["offers"]:
            providers.update(
                {
                    entry["provider_id"]: {
                        "shortname": entry["package_short_name"],
                    }
                }
            )
    except KeyError:
        # This means that there are no providers found in the configured locale.
        # Ignore this exception and return an empty dict.
        pass

    return providers


def get_release_date(raw_movie_data, format="%Y-%m-%d"):
    release_cinema = raw_movie_data.get("inCinemas")
    release_digital = raw_movie_data.get("digitalRelease")
    release_physical = raw_movie_data.get("physicalRelease")

    if release_cinema:
        release_date = datetime.datetime.strptime(release_cinema, "%Y-%m-%dT%H:%M:%SZ").strftime(
            format
        )
    elif release_digital:
        release_date = datetime.datetime.strptime(release_digital, "%Y-%m-%dT%H:%M:%SZ").strftime(
            format
        )
    elif release_physical:
        release_date = datetime.datetime.strptime(release_physical, "%Y-%m-%dT%H:%M:%SZ").strftime(
            format
        )
    else:
        release_date = None

    return release_date


def get_filesize_gb(filesize):
    filesize_gb = filesize / 1024.0 ** 3
    return "%.2f" % filesize_gb + "GB"


def get_episode_data(episodes, season_number, episode_number):
    episode_data = {}

    for episode in episodes:
        if season_number == episode["seasonNumber"] and episode_number == episode["episodeNumber"]:
            episode_data = {
                "episode_id": episode["id"],
                "monitored": episode["monitored"],
                "has_file": episode.get("hasFile", False),
            }
            break

    return episode_data


def get_episode_file_id(episodes, season_number, episode_number):
    episode_file_ids = []

    for episode in episodes:
        if (
            season_number == episode["seasonNumber"]
            and episode_number == episode["episodeNumber"]
            and episode.get("hasFile", False)
        ):
            episode_file_ids.append(episode["episodeFileId"])

    return episode_file_ids


def get_pretty_seasons(seasons):
    season_data = []

    for season in seasons:
        season_number = season["season"]
        season_data.append(f"Season {season_number}")

    return ", ".join(season_data)


def get_pretty_episodes(episodes):
    episode_data = []

    for episode in episodes:
        season_number = episode["season"]
        episode_number = episode["episode"]
        episode_data.append(f"S{season_number:02d}E{episode_number:02d}")

    return ", ".join(episode_data)


def get_providers_from_seasons_episodes(seasons, episodes):
    season_providers = flatten(
        [season["providers"] for season in seasons if season.get("providers")]
    )
    episode_providers = flatten(
        [episode["providers"] for episode in episodes if episode.get("providers")]
    )

    providers = list(set(season_providers + episode_providers))

    return ", ".join(providers)


def modify_sonarr_seasons(sonarr_object, seasons, monitored):
    sonarr_seasons = sonarr_object["seasons"]

    for entry in sonarr_seasons:
        if entry["seasonNumber"] in seasons:
            entry["monitored"] = monitored

    sonarr_object["seasons"] = sonarr_seasons

    return sonarr_object



#!/usr/bin/env python

import requests

from json import JSONDecodeError

from .exceptions import (
    SonarrInvalidIdSupplied,
    SonarrInvalidApiKey,
    SonarrMovieNotFound,
    SonarrValidationException,
)
from .v3.serie import Serie
from .v3.episode import Episode
from .v3.episodefile import EpisodeFile


class Sonarr(object):
    def __init__(self, base_url, api_key, ssl_verify=True):
        # Setup base variables
        self._base_url = base_url.rstrip("/")
        self.api_key = api_key
        self.api_url = "{}/api/v3".format(self._base_url)

        # Setup session
        self.session = requests.Session()
        self.session.verify = ssl_verify
        self.session.headers.update({"X-Api-Key": self.api_key})

        # Register managers based on api version
        self.serie = Serie(self)
        self.episode = Episode(self)
        self.episodefile = EpisodeFile(self)

    def __exit__(self, *args):
        self.session.close()

    def _build_url(self, path):
        return "{}{}".format(self.api_url, path)

    def _filter_api_error(self, data):
        if data.status_code == 400:
            raise SonarrInvalidIdSupplied(
                "Invalid ID supplied! The error message is: {}".format(data.text)
            )
        elif data.status_code == 401:
            raise SonarrInvalidApiKey("Invalid API key")
        elif data.status_code == 404:
            raise SonarrMovieNotFound("Serie not found")
        elif data.status_code == 405:
            raise SonarrValidationException("Validation exception")

        result_json = data.json()

        return result_json

    def http_request(self, method, path, json=None, params=None, retries=3):
        url = self._build_url(path)
        request = requests.Request(method, url, json=json, params=params)

        prepped = self.session.prepare_request(request)
        result = self.session.send(prepped)

        try:
            return self._filter_api_error(result)
        except ValueError:
            if retries != 0:
                retries -= 1
                self.http_request(method, path, json, params, retries)
            else:
                raise

    def http_get(self, path, params=None):
        return self.http_request("get", path, params=params)

    def http_post(self, path, json=None):
        return self.http_request("post", path, json=json)

    def http_put(self, path, params=None, json=None):
        return self.http_request("put", path, params=params, json=json)

    def http_delete(self, path, json=None, params=None):
        return self.http_request("delete", path, json=json, params=params)



import rich
import typer

from typing import List, Optional
from loguru import logger

import excludarr.utils.output as output

from excludarr.core.radarr_actions import RadarrActions
from excludarr.utils.config import Config
from excludarr.utils.enums import Action

app = typer.Typer()


@app.command(help="Exclude movies in Radarr by deleting or not monitoring them")
def exclude(
    providers: Optional[List[str]] = typer.Option(
        None,
        "-p",
        "--provider",
        metavar="PROVIDER",
        help="Override the configured streaming providers.",
    ),
    locale: Optional[str] = typer.Option(
        None, "-l", "--locale", metavar="LOCALE", help="Your locale e.g: en_US."
    ),
    action: Action = typer.Option(..., "-a", "--action", help="Change the status in Radarr."),
    delete_files: bool = typer.Option(
        False, "-d", "--delete-files", help="Delete already downloaded files."
    ),
    exclusion: bool = typer.Option(
        False, "-e", "--exclusion", help="Add an exclusion to prevent auto importing."
    ),
    yes: bool = typer.Option(False, "-y", "--yes", help="Auto accept the confirmation notice."),
    progress: bool = typer.Option(
        False, "--progress", help="Track the progress using a progressbar."
    ),
):
    """
    Radarr exclude function. This function handles the CLI input and determines which
    action (delete/not-monitored) should be executed. The function gathers all information
    from Radarr and checks if there is a match with the JustWatch API. Based on the results a
    table is printed and the user is asked for confirmation. Based on the CLI options the
    files in Radarr are also deleted.

    :param providers: A list of providers that should overwrite the configuration settings
    :param locale: The locale in en_US or two letter country code US
    :param action: The action that should be executed, either delete or not-monitored
    :param delete_files: Also delete the already downloaded movie files
    :param exclusion: Add import exclusion to prevent auto importing the movie again
    :param yes: Skip the confirmation notice and continue without user input
    :param progress: Show a progress bar, only works when no --debug flag is set
    :return: None
    """
    # Debug logging
    logger.debug("Got exclude as subcommand")
    logger.debug(f"Got CLI values for -p, --provider option: {', '.join(providers)}")
    logger.debug(f"Got CLI values for -l, --locale option: {locale}")
    logger.debug(f"Got CLI values for -a, --action option: {action}")
    logger.debug(f"Got CLI values for -d, --delete option: {delete_files}")
    logger.debug(f"Got CLI values for -e, --exclusion option: {exclusion}")
    logger.debug(f"Got CLI values for -y, --yes option: {yes}")
    logger.debug(f"Got CLI values for --progress option: {progress}")

    # Disable the progress bar when debug logging is active
    if loglevel == 10:
        disable_progress = True
    elif progress and loglevel != 10:
        disable_progress = False
    else:
        disable_progress = True

    # Determine if CLI options should overwrite configuration settings
    if not providers:
        providers = config.providers
    if not locale:
        locale = config.locale

    # Setup Radarr Actions to control the different tasks
    radarr = RadarrActions(
        config.radarr_url, config.radarr_api_key, locale, config.radarr_verify_ssl
    )

    # Get the movies to exclude and exclude the movies that are in the exclude list
    movies_to_exclude = radarr.get_movies_to_exclude(
        providers, config.fast_search, disable_progress
    )

    # Only take monitored movies when the action is not-monitored
    if action == Action.not_monitored:
        movies_to_exclude = {
            id: values
            for id, values in movies_to_exclude.items()
            if values["title"] not in config.radarr_excludes
            and values["radarr_object"]["monitored"]
        }
    else:
        movies_to_exclude = {
            id: values
            for id, values in movies_to_exclude.items()
            if values["title"] not in config.radarr_excludes
        }

    # Create a list of the Radarr IDs
    movies_to_exclude_ids = list(movies_to_exclude.keys())

    # If there are movies to exclude
    if movies_to_exclude_ids:
        # Calculate total filesize
        total_filesize = sum([movie["filesize"] for _, movie in movies_to_exclude.items()])

        # Print the movies in table format
        output.print_movies_to_exclude(movies_to_exclude, total_filesize)

        # Check for confirmation
        if not yes:
            confirmation = output.ask_confirmation(action, "movies")
            if not confirmation:
                logger.warning("Aborting Excludarr because user did not confirm the question")
                raise typer.Abort()
        else:
            confirmation = True

        if confirmation:
            # Determine and execute the action supplied (delete, not-monitored)
            if action == Action.delete:
                radarr.delete(movies_to_exclude_ids, delete_files, exclusion)
            elif action == Action.not_monitored:
                movie_info = [movie["radarr_object"] for _, movie in movies_to_exclude.items()]
                radarr.disable_monitored(movie_info)

                if delete_files:
                    radarr.delete_files(movies_to_exclude_ids)

            output.print_success_exclude(action, "movies")
    else:
        rich.print("There are no more movies also available on the configured streaming providers!")


@app.command(help="Change status of movies to monitored if no provider is found")
def re_add(
    providers: Optional[List[str]] = typer.Option(
        None,
        "-p",
        "--provider",
        metavar="PROVIDER",
        help="Override the configured streaming providers.",
    ),
    locale: Optional[str] = typer.Option(
        None, "-l", "--locale", metavar="LOCALE", help="Your locale e.g: en_US."
    ),
    yes: bool = typer.Option(False, "-y", "--yes", help="Auto accept the confirmation notice."),
    progress: bool = typer.Option(
        False, "--progress", help="Track the progress using a progressbar."
    ),
):
    # Debug logging
    logger.debug("Got re-add as subcommand")
    logger.debug(f"Got CLI values for -p, --provider option: {', '.join(providers)}")
    logger.debug(f"Got CLI values for -l, --locale option: {locale}")
    logger.debug(f"Got CLI values for -y, --yes option: {yes}")
    logger.debug(f"Got CLI values for --progress option: {progress}")

    # Disable the progress bar when debug logging is active
    if loglevel == 10:
        disable_progress = True
    elif progress and loglevel != 10:
        disable_progress = False
    else:
        disable_progress = True

    # Determine if CLI options should overwrite configuration settings
    if not providers:
        providers = config.providers
    if not locale:
        locale = config.locale

    # Setup Radarr Actions to control the different tasks
    radarr = RadarrActions(
        config.radarr_url, config.radarr_api_key, locale, config.radarr_verify_ssl
    )

    # Get the movies that should be re monitored
    movies_to_re_add = radarr.get_movies_to_re_add(providers, config.fast_search, disable_progress)
    movies_to_re_add = {
        id: values
        for id, values in movies_to_re_add.items()
        if values["title"] not in config.radarr_excludes
    }

    # Create a list of the Radarr IDs
    movies_to_re_add_ids = list(movies_to_re_add.keys())

    # If there are movies to exclude
    if movies_to_re_add_ids:
        # Print the movies in table format
        output.print_movies_to_re_add(movies_to_re_add)

        # Check for confirmation
        if not yes:
            confirmation = output.ask_confirmation("re-add", "movies")
            if not confirmation:
                logger.warning("Aborting Excludarr because user did not confirm the question")
                raise typer.Abort()
        else:
            confirmation = True

        if confirmation:
            # Re-add the movies
            movie_info = [movie["radarr_object"] for _, movie in movies_to_re_add.items()]
            radarr.enable_monitored(movie_info)

            rich.print(
                "Succesfully changed the status of the movies listed in Radarr to monitored!"
            )

    else:
        rich.print("There are no more movies to re-add!")


@app.callback()
def init():
    """
    Initializes the command. Reads the configuration.
    """
    logger.debug("Got radarr as subcommand")

    # Set globals
    global config
    global loglevel

    # Hacky way to get the current log level context
    loglevel = logger._core.min_level

    logger.debug("Reading configuration file")
    config = Config()


if __name__ == "__main__":
    app()



import pathlib
from setuptools import setup, find_packages
from excludarr import __version__

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="excludarr",
    version=__version__,
    description="Excludarr excludes streaming services from Radarr and Sonarr",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/haijeploeg/excludarr",
    author="Haije Ploeg",
    author_email="ploeg.haije@gmail.com",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
    ],
    license="MIT",
    keywords="excludarr, radarr, sonarr, diskspace, cleanup, management, library, streaming, netflix",
    packages=find_packages(exclude=["ez_setup", "tests*"]),
    python_requires=">=3.6, <4",
    install_requires=[
        "typer>=0.4.0",
        "loguru>=0.5.3",
        "rich>=10.16.1",
        "requests>=2.26.0",
        "pyyaml>=6.0",
    ],
    entry_points={
        "console_scripts": [
            "excludarr=excludarr.main:cli",
        ],
    },
    project_urls={
        "Bug Reports": "https://github.com/haijeploeg/excludarr/issues",
        "Source": "https://github.com/haijeploeg/excludarr",
    },
)



#!/usr/bin/env python

from ..base import Manager


class Movie(Manager):
    def __init__(self, client):
        super().__init__(client)
        self.movie_path = "/movie"
        self.movie_id_path = "/movie/{id}".format(id="{id}")
        self.movie_lookup_path = "/movie/lookup"
        self.movie_editor_path = "/movie/editor"
        self.movie_import_path = "/movie/import"

    def get_all_movies(self):
        return self.client.http_get(self.movie_path)

    def get_movie_by_tmdbid(self, tmdb_id):
        params = {"tmdbid": tmdb_id}
        return self.client.http_get(self.movie_path, params=params)

    def add_movie(
        self,
        title,
        quality_profile_id,
        title_slug,
        images,
        tmdb_id,
        year,
        root_folder_path,
        path=None,
        monitored=True,
        search_for_movie=False,
    ):
        """
        TODO: figure out how to do this according to the documentation: https://radarr.video/docs/api/#/Movie/post_movie
        This is the old format, which works, but does not comply with the API v3 fomat.
        """

        json = {
            "title": title,
            "qualityProfileId": quality_profile_id,
            "titleSlug": title_slug,
            "images": images,
            "tmdbId": tmdb_id,
            "year": year,
            "path": path,
            "rootFolderPath": root_folder_path,
            "monitored": monitored,
            "addOptions": {"searchForMovie": search_for_movie},
        }
        return self.client.http_post(self.movie_path, json=json)

    def update_movie(self, updated_movie_dict, move_files=False):
        params = {"moveFiles": move_files}
        return self.client.http_put(
            self.movie_path, params=params, json=updated_movie_dict
        )

    def get_movie_by_id(self, id):
        return self.client.http_get(self.movie_id_path.format(id=id))

    def delete_movie(self, id, delete_files=False, add_import_exclusion=False):
        params = {
            "deleteFiles": delete_files,
            "addImportExclusion": add_import_exclusion,
        }
        return self.client.http_delete(self.movie_id_path.format(id=id), params=params)

    def lookup_movie(self, term):
        params = {"term": term}
        return self.client.http_get(self.movie_lookup_path, params=params)

    def update_movies(self, **kwargs):
        return self.client.http_put(self.movie_path, json=kwargs)

    def delete_movies(self, movie_ids, delete_files=False, add_import_exclusion=False):
        json = {
            "movieIds": movie_ids,
            "deleteFiles": delete_files,
            "addImportExclusion": add_import_exclusion,
        }
        return self.client.http_delete(self.movie_editor_path, json=json)

    def add_movies(self):
        """
        TODO: figure out how to do this properly
        """
        pass



#!/usr/bin/env python

# Imports
import requests

from json import JSONDecodeError

from .exceptions import TMDBException
from .v3.movies import Movie
from .v3.tv import TV
from .v3.find import Find


class TMDB(object):
    def __init__(self, api_key, api_version="3", ssl_verify=True):
        # Setup base variables
        self._api_version = str(api_version)
        self._base_url = "https://api.themoviedb.org"
        self.api_url = "{}/{}".format(self._base_url, self._api_version)
        self.api_key = api_key

        # Setup session
        self.session = requests.Session()
        self.session.verify = ssl_verify
        self.session.params = {"api_key": self.api_key}

        # Register managers
        self.movie = Movie(self)
        self.tv = TV(self)
        self.find = Find(self)

    def __exit__(self, *args):
        self.session.close()

    def _build_url(self, path):
        return "{}{}".format(self.api_url, path)

    def _filter_api_error(self, data):
        if not data.get("success", True):
            status_code = data.get("status_code", "")
            status_message = data.get("status_message", "")
            raise TMDBException(status_code, status_message)

        return data

    def http_request(self, method, path, json=None, params=None):
        url = self._build_url(path)
        request = requests.Request(method, url, json=json, params=params)

        prepped = self.session.prepare_request(request)
        result = self.session.send(prepped)

        try:
            result_json = result.json()
        except JSONDecodeError:
            return result.text

        return self._filter_api_error(result_json)

    def http_get(self, path, params=None):
        return self.http_request("get", path, params=params)
